@@ -26,7 +26,7 @@ In the PyAutomation KPart, the following can be used to get a qtxmldom version
 of the active document:
 
   import qtxmldom
-  doc = fromKHTML(part.document())
+  doc = fromNode(part.document())
 
 If the module is edited, then the reload built-in function will probably update
 the code, but the active DOM objects then need to be discarded.
@@ -34,7 +34,7 @@ the code, but the active DOM objects then need to be discarded.
 
 import xml.dom
 
-__version__ = "0.2"
+__version__ = "0.2.1"
 
 class Implementation(object):
 
@@ -99,7 +99,7 @@ class khtmlImplementation(Implementation):
 
 class qtxmlImplementation(Implementation):
 
-    def __init__(self, _impl):
+    def __init__(self, _impl=None):
         import qtxml, qt
         self.qtxml = qtxml
         self.qt = qt
@@ -384,16 +384,34 @@ class Node(object):
         return NamedNodeMap(self._node.attributes(), self.impl)
 
     def getAttributeNS(self, ns, localName):
-        return self.impl.get_string(self._node.getAttributeNS(self.impl.make_string(ns), self.impl.make_string(localName)))
+        if hasattr(self._node, "getAttributeNS"):
+            return self.impl.get_string(self._node.getAttributeNS(self.impl.make_string(ns), self.impl.make_string(localName)))
+        else:
+            return self.impl.get_string(self._node.attributeNS(self.impl.make_string(ns), self.impl.make_string(localName)))
 
     def getAttribute(self, name):
-        return self.impl.get_string(self._node.getAttribute(self.impl.make_string(name)))
+        if hasattr(self._node, "getAttribute"):
+            return self.impl.get_string(self._node.getAttribute(self.impl.make_string(name)))
+        else:
+            return self.impl.get_string(self._node.attribute(self.impl.make_string(name)))
 
     def getAttributeNodeNS(self, ns, localName):
-        return Node(self._node.getAttributeNodeNS(self.impl.make_string(ns), self.impl.make_string(localName)), self.impl)
+        if hasattr(self._node, "getAttributeNodeNS"):
+            return Node(self._node.getAttributeNodeNS(self.impl.make_string(ns), self.impl.make_string(localName)), self.impl)
+        else:
+            return Node(self._node.attributeNodeNS(self.impl.make_string(ns), self.impl.make_string(localName)), self.impl)
 
     def getAttributeNode(self, name):
-        return Node(self._node.getAttribute(self.impl.make_string(name)), self.impl)
+        if hasattr(self._node, "getAttributeNode"):
+            return Node(self._node.getAttributeNode(self.impl.make_string(name)), self.impl)
+        else:
+            return Node(self._node.attributeNode(self.impl.make_string(name)), self.impl)
+
+    def hasAttributeNS(self, ns, name):
+        return self._node.hasAttributeNS(self.impl.make_string(ns), self.impl.make_string(name))
+
+    def hasAttribute(self, name):
+        return self._node.hasAttribute(self.impl.make_string(name))
 
     def setAttributeNS(self, ns, name, value):
         self._node.setAttributeNS(self.impl.make_string(ns), self.impl.make_string(name), self.impl.make_string(value))
@@ -466,10 +484,16 @@ class Node(object):
         return Node(self._node.appendChild(node._node), self.impl)
 
     def getElementsByTagName(self, tagName):
-        return NodeList(self._node.getElementsByTagName(self.impl.make_string(tagName)), self.impl)
+        if hasattr(self._node, "getElementsByTagName"):
+            return NodeList(self._node.getElementsByTagName(self.impl.make_string(tagName)), self.impl)
+        else:
+            return NodeList(self._node.elementsByTagName(self.impl.make_string(tagName)), self.impl)
 
     def getElementsByTagNameNS(self, namespaceURI, localName):
-        return NodeList(self._node.getElementsByTagNameNS(self.impl.make_string(namespaceURI), self.impl.make_string(localName)), self.impl)
+        if hasattr(self._node, "getElementsByTagNameNS"):
+            return NodeList(self._node.getElementsByTagNameNS(self.impl.make_string(namespaceURI), self.impl.make_string(localName)), self.impl)
+        else:
+            return NodeList(self._node.elementsByTagNameNS(self.impl.make_string(namespaceURI), self.impl.make_string(localName)), self.impl)
 
     def normalize(self):
         self._node.normalize()
@@ -498,7 +522,16 @@ class Node(object):
         if hasattr(self._node, "toString"):
             return self.impl.get_string(self._node.toString())
         else:
-            return qtxmlImplementation().get_string(self._node.toHTML())
+            # NOTE: Verify encodings!
+
+            import qt
+            byte_array = qt.QByteArray()
+            ostream = qt.QTextOStream(byte_array)
+            ostream.setEncoding(qt.QTextStream.Unicode)
+            self._node.save(ostream, 2)
+            istream = qt.QTextIStream(byte_array)
+            istream.setEncoding(qt.QTextStream.Unicode)
+            return unicode(istream.read())
 
     def toString(self, encoding="utf-8"):
         return self.toUnicode().encode(encoding)
@@ -517,4 +550,54 @@ def fromQDomNode(node):
 
     return Node(node, qtxmlImplementation())
 
+def parse(stream_or_string):
+
+    """
+    Parse the given 'stream_or_string', where the supplied object can either be
+    a stream (such as a file or stream object), or a string (containing the
+    filename of a document).
+
+    A document object is returned by this function.
+    """
+
+    if hasattr(stream_or_string, "read"):
+        stream = stream_or_string
+        return parseString(stream.read())
+    else:
+        return parseFile(stream_or_string)
+
+def parseFile(filename):
+
+    """
+    Parse the file having the given 'filename'.
+
+    A document object is returned by this function.
+    """
+
+    import qt, qtxml
+    f = qt.QFile(filename)
+    try:
+        d = qtxml.QDomDocument()
+        if d.setContent(f)[0]:
+            return fromQDomNode(d)
+        else:
+            raise IOError, filename
+    finally:
+        f.close()
+
+def parseString(s):
+
+    """
+    Parse the content of the given string 's'.
+
+    A document object is returned by this function.
+    """
+
+    import qtxml
+    d = qtxml.QDomDocument()
+    if d.setContent(s)[0]:
+        return fromQDomNode(d)
+    else:
+        raise IOError, s
+
 # vim: tabstop=4 expandtab shiftwidth=4
